import json

from flask import Flask
from redis import Redis

from occam.runtime import OCCAM_SERVER_CONFIG_KEY
from occam.runtime import parse_config

app = Flask(__name__)


def attach_occam_config_to_app(config_path):
    global app
    config = parse_config(config_path)
    app.config.update(**config)
    redis = get_redis()
    redis.set(OCCAM_SERVER_CONFIG_KEY, json.dumps(config['razor_servers']))
    return config


def get_redis():
    """Creates a new Redis connection based on configuration passed to the app.

    I know this is hacky and I should take the time to implement this better,
    but it works for the time being :)
    """
    redis_config = app.config.get('redis_config', {})
    return Redis(host=redis_config.get('host', 'localhost'),
                 port=redis_config.get('port', 6379),
                 db=redis_config.get('db', 0))


import occam.views



import json

from occam.app import get_redis


def make_key(server_name, *key):
    key_data = ["occam", "data", server_name]
    key_data.extend(key)
    return ":".join(key_data)


def put_json_data(key, value):
    redis = get_redis()
    redis.set(key, json.dumps(value))


def replace_list(key, values):
    redis = get_redis()
    pipe = redis.pipeline()
    pipe.delete(key)
    for item in values:
        pipe.lpush(key, json.dumps(item))
    pipe.execute()



from optparse import OptionParser
import os.path
import sys

import yaml


OCCAM_SERVER_CONFIG_KEY = "occam:config:servers"


def build_parser():
    parser = OptionParser()
    parser.add_option("-c", "--config", help="Path to YAML config file")
    return parser


def parse_args(parser):
    parser = parser
    return parser.parse_args()


def acquire_runtime_args(parser=None):
    parser = parser or build_parser()
    opts, args = parser.parse_args()
    if not opts.config:
        print >> sys.stderr, "--config option is required!"
        sys.exit(1)
    if not os.path.exists(opts.config):
        print >> sys.stderr, "Could not open configuration at %s" % opts.config
        sys.exit(1)

    return opts, args


def parse_config(config_path):
    with open(config_path) as f:
        return yaml.load(f.read())


def make_redis_url(redis_config):
    redis_host = redis_config.get("host", "localhost")
    redis_port = redis_config.get("port", 6379)
    redis_db = redis_config.get("db", 0)
    return "redis://%s:%s/%s" % (redis_host, redis_port, redis_db)



import functools
import json

from flask import jsonify
from flask import redirect
from flask import render_template
from flask import request
from flask import url_for

from occam.app import app
from occam.app import get_redis
from occam.data import make_key
from occam.util import iterate_servers
from occam.version import VERSION


def json_or_template(template_name):
    def decorator(f):
        @functools.wraps(f)
        def wrapper(*args, **kwargs):
            result = f(*args, **kwargs)
            if "application/json" in request.headers['Accept']:
                return jsonify(**result)
            else:
                result.update(version=VERSION)
                return render_template(template_name, **result)
        return wrapper
    return decorator


def collection_view(collection, server=None, item=None):
    redis = get_redis()
    items = {}
    for server_name, _ in iterate_servers():
        server_item_refs = json.loads(redis.get(make_key(server_name, collection)))
        server_item_names = map(lambda x: x['name'], server_item_refs)
        server_items = dict((server_item, json.loads(redis.get(make_key(server_name, collection, server_item))))
                            for server_item in server_item_names)
        items[server_name] = server_items

    result = {
        "items": items,
        "collection": collection
    }
    if server and item:
        result.update({"server": server, "selected": item})
    return result


@app.route("/")
def index():
    return redirect(url_for('activity'))


@app.route("/activity")
@app.route("/activity/<server>/<node>")
@json_or_template("activity.html")
def activity(server=None, node=None):
    redis = get_redis()
    start = request.args.get('start', 0)
    end = request.args.get('end', 99)
    if not server and not node:
        history_key = make_key("_all", "history")
    else:
        history_key = make_key(server, node, "log")
    max_entry = redis.llen(history_key)

    entries_raw = redis.lrange(history_key, start, end)
    entries = map(json.loads, entries_raw)
    nodes = {}
    for server_name, _ in iterate_servers():
        server_entries = filter(lambda x: x['server'] == server_name, entries)
        server_node_refs = set(map(lambda x: x['node'], server_entries))
        server_nodes = dict((node, json.loads(redis.get(make_key(server_name, "nodes", node))))
                            for node in server_node_refs)
        nodes[server_name] = server_nodes
    return {
        "entries": entries,
        "nodes": nodes,
        "start": start,
        "end": end,
        "max_entry": max_entry
    }


@app.route("/nodes")
@app.route("/nodes/<server>/<node>")
@app.route("/node/<server>/<node>")
@json_or_template("collection.html")
def nodes(server=None, node=None):
    result = collection_view("nodes", server, node)
    return result


@app.route("/policies")
@app.route("/policies/<server>/<policy>")
@app.route("/policy/<server>/<policy>")
@json_or_template("collection.html")
def policies(server=None, policy=None):
    result = collection_view("policies", server, policy)
    return result


@app.route("/tags")
@app.route("/tags/<server>/<tag>")
@app.route("/tag/<server>/<tag>")
@json_or_template("collection.html")
def tags(server=None, tag=None):
    result = collection_view("tags", server, tag)
    return result


@app.route("/repos")
@app.route("/repos/<server>/<repo>")
@app.route("/repo/<server>/<repo>")
@json_or_template("collection.html")
def repos(server=None, repo=None):
    result = collection_view("repos", server, repo)
    return result


@app.route("/brokers")
@app.route("/brokers/<server>/<broker>")
@app.route("/broker/<server>/<broker>")
@json_or_template("collection.html")
def brokers(server=None, broker=None):
    result = collection_view("brokers", server, broker)
    return result



import json

from dateutil import parser as datetime_parser

from occam.app import get_redis
from occam.runtime import OCCAM_SERVER_CONFIG_KEY


def get_servers():
    redis = get_redis()
    servers = json.loads(redis.get(OCCAM_SERVER_CONFIG_KEY))

    return servers.items()


def iterate_servers():
    redis = get_redis()
    servers = json.loads(redis.get(OCCAM_SERVER_CONFIG_KEY))

    for server_name, server_location in servers.iteritems():
        yield server_name, server_location


def sorted_by_time_element(l, element_getter=None):
    if not element_getter:
        element_getter = lambda x: x
    key_getter = lambda x: datetime_parser.parse(element_getter(x))
    return sorted(l, key=key_getter)



# -*- coding: utf-8 -*-
from setuptools import setup

from occam.version import VERSION

setup(
    name="occam",
    version=VERSION,
    author="Fred Hatfull",
    author_email="fred.hatfull@gmail.com",
    description=("A web frontend for the Razor imaging server."),
    license="MIT",
    keywords="razor razor-server imaging web",
    url="https://github.com/fhats/occam",
    packages=['occam', 'tests'],
    long_description="A frontend to the Razor automated imager tool. See https://github.com/fhats/occam for more information.",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Systems Administration",
        "Topic :: System :: Installation/Setup",
    ],
    install_requires=[
        "celery[redis] == 3.1.9",
        "python-dateutil == 2.2",
        "Flask == 0.10.1",
        "requests == 2.2.0",
        "py_razor_client == 0.1.2",
        "pyyaml == 3.10",
    ],
    tests_require=[
        "coverage == 3.7.1",
        "mock == 1.0.1",
        "testify == 0.5.2"
    ]
)
